@@ -58,6 +58,7 @@ void FetchHelper::init()
 {
   mAncestorDepth = 0;
   mCacheOnly = false;
+  mCheckCachedPayloadPartsOnly = false;
   mFullPayload = false;
   mAllAttrs = false;
   mSizeRequested = false;
@@ -315,8 +316,11 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
     if ( mAncestorDepth > 0 )
       attributes.append( HandlerHelper::ancestorsToByteArray( mAncestorDepth, ancestorsForItem( parentCollectionId ) ) );
 
+
     bool skipItem = false;
 
+    QList<QByteArray> cachedParts;
+
     while ( partQuery.isValid() ) {
       const qint64 id = partQuery.value( PartQueryPimIdColumn ).toLongLong();
       if ( id > pimItemId ) {
@@ -328,35 +332,43 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       QByteArray partName = Utils::variantToByteArray( partQuery.value( PartQueryNameColumn ) );
       QByteArray part = Utils::variantToByteArray( partQuery.value( PartQueryNameColumn ) );
       QByteArray data = Utils::variantToByteArray( partQuery.value( PartQueryDataColumn ) );
-      if ( mIgnoreErrors && data.isEmpty() ) {
-        //We wanted the payload, couldn't get it, and are ignoring errors. Skip the item.
-        akDebug() << id << " no payload";
-        skipItem = true;
-        break;
-      }
-      bool partIsExternal = partQuery.value( PartQueryExternalColumn ).toBool();
-      if ( !mExternalPayloadSupported && partIsExternal ) //external payload not supported by the client, translate the data
-        data = PartHelper::translateData( data, partIsExternal );
-      int version = partQuery.value( PartQueryVersionColumn ).toInt();
-      if ( version != 0 ) { // '0' is the default, so don't send it
-        part += '[' + QByteArray::number( version ) + ']';
-      }
-      if (  mExternalPayloadSupported && partIsExternal ) { // external data and this is supported by the client
-        part += " [FILE] ";
-      }
-      if ( data.isNull() ) {
-        part += " NIL";
-      } else if ( data.isEmpty() ) {
-        part += " \"\"";
-      } else {
-        part += " {" + QByteArray::number( data.length() ) + "}\r\n";
-        part += data;
-      }
 
-      if ( mRequestedParts.contains( partName ) || mFullPayload || mAllAttrs )
-        attributes << part;
+      if ( mCheckCachedPayloadPartsOnly ) {
+        if ( !data.isEmpty() ) {
+           cachedParts << part;
+        }
+         partQuery.next();
+     } else {
+        if ( mIgnoreErrors && data.isEmpty() ) {
+          //We wanted the payload, couldn't get it, and are ignoring errors. Skip the item.
+          akDebug() << id << " no payload";
+          skipItem = true;
+          break;
+        }
+        bool partIsExternal = partQuery.value( PartQueryExternalColumn ).toBool();
+        if ( !mExternalPayloadSupported && partIsExternal ) //external payload not supported by the client, translate the data
+          data = PartHelper::translateData( data, partIsExternal );
+        int version = partQuery.value( PartQueryVersionColumn ).toInt();
+        if ( version != 0 ) { // '0' is the default, so don't send it
+          part += '[' + QByteArray::number( version ) + ']';
+        }
+        if (  mExternalPayloadSupported && partIsExternal ) { // external data and this is supported by the client
+          part += " [FILE] ";
+        }
+        if ( data.isNull() ) {
+          part += " NIL";
+        } else if ( data.isEmpty() ) {
+          part += " \"\"";
+        } else {
+          part += " {" + QByteArray::number( data.length() ) + "}\r\n";
+          part += data;
+        }
 
-      partQuery.next();
+        if ( mRequestedParts.contains( partName ) || mFullPayload || mAllAttrs )
+          attributes << part;
+
+        partQuery.next();
+      }
     }
 
     if (skipItem) {
@@ -364,6 +376,11 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       continue;
     }
 
+    if ( mCheckCachedPayloadPartsOnly ) {
+      attributes.append( "CACHEDPARTS (" + ImapParser::join( cachedParts, " " ) + ')');
+    }
+
+
     // IMAP protocol violation: should actually be the sequence number
     QByteArray attr = QByteArray::number( pimItemId ) + ' ' + responseIdentifier + " (";
     attr += ImapParser::join( attributes, " " ) + ')';
@@ -375,12 +392,21 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
   }
 
   // update atime (only if the payload was actually requested, otherwise a simple resource sync prevents cache clearing)
-  if ( !payloadList.isEmpty() || mFullPayload )
+  if ( needsAccessTimeUpdate(partList) || mFullPayload )
     updateItemAccessTime();
 
   return true;
 }
 
+bool FetchHelper::needsAccessTimeUpdate(const QStringList& parts)
+{
+  // TODO technically we should compare the part list with the cache policy of
+  // the parent collection of the retrieved items, but that's kinda expensive
+  // Only updating the atime if the full payload was requested is a good
+  // approximation though.
+  return parts.contains( QLatin1String("PLD:RFC822") );
+}
+
 void FetchHelper::updateItemAccessTime()
 {
   Transaction transaction( mConnection->storageBackend() );
@@ -427,6 +453,8 @@ void FetchHelper::parseCommandStream()
       const QByteArray buffer = mStreamParser->readString();
       if ( buffer == AKONADI_PARAM_CACHEONLY ) {
         mCacheOnly = true;
+      } else if ( buffer == AKONADI_PARAM_CHECKCACHEDPARTSONLY ) {
+        mCheckCachedPayloadPartsOnly = true;
       } else if ( buffer == AKONADI_PARAM_ALLATTRIBUTES ) {
         mAllAttrs = true;
       } else if ( buffer == AKONADI_PARAM_EXTERNALPAYLOAD ) {
